/*
	gl_funcs.c

	GL functions.

	Copyright (C) 1996-1997  Id Software, Inc.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to:

		Free Software Foundation, Inc.
		59 Temple Place - Suite 330
		Boston, MA  02111-1307, USA

	$Id$
*/

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#ifdef HAVE_DLFCN_H
#include <dlfcn.h>
#endif
#ifdef HAVE_STRING_H
# include <string.h>
#endif
#ifdef HAVE_STRINGS_H
# include <strings.h>
#endif
#include <stdio.h>

#include <QF/GL/types.h>
#include <QF/GL/funcs.h>
#include <QF/GL/extensions.h>
#include <QF/cvar.h>
#include <QF/console.h>
#include <QF/sys.h>
#include "r_cvar.h"

// First we need to get all the function pointers declared.
#define QFGL_NEED(ret, name, args) \
ret (* name) args = NULL;
#include "QF/GL/qf_funcs_list.h"
#undef QFGL_NEED

// Then we need to open the libGL and set all the symbols.
qboolean
GLF_Init (void)
{
	void	*handle;

#ifdef HAVE_DLOPEN
	if (!(handle = dlopen (gl_libgl->string, RTLD_NOW))) {
		Sys_Error ("Couldn't load OpenGL library %s: %s\n", gl_libgl->string, dlerror ());
		return false;
	}
#else
# if _WIN32
	if (!(handle = LoadLibrary (gl_libgl->string))) {
		Sys_Error ("Couldn't load OpenGL library %s!\n", gl_libgl->string);
		return false;
	}
# else
	Sys_Error ("Cannot load libraries: %s was built without DSO support", PROGRAM);
	return false;
# endif
#endif

#define QFGL_NEED(ret, name, args)	\
	name = QFGL_ProcAddress (handle, #name);

#include "QF/GL/qf_funcs_list.h"
#undef QFGL_NEED
	QFGL_ProcAddress (NULL, NULL); // tell ProcAddress to clear its cache

	return true;
}

void *
QFGL_ProcAddress (void *handle, const char *name)
{
	static qboolean inited = false;
	void			*glfunc = NULL;

#ifdef HAVE_DLOPEN
	static QF_glXGetProcAddressARB	glGetProcAddress = NULL;
#else
# ifdef _WIN32
	static void * (* wglGetProcAddress)	(char *)	glGetProcAddress = NULL;
# endif
#endif

	if (!handle || !name)
		return NULL;

	if (!inited) {
		inited = true;
#ifdef HAVE_DLOPEN
		glGetProcAddress = dlsym (handle, "glXGetProcAddressARB");
#else
# ifdef _WIN32
		glGetProcAddress = GetProcAddress (handle, "wglGetProcAddress");
# endif
#endif
	}

	Sys_Printf ("DEBUG: Finding symbol %s ... ", name);
#ifdef HAVE_DLOPEN
	if (glGetProcAddress)
		glfunc = glGetProcAddress (name);
	else
		glfunc = dlsym (handle, name);
#else
# ifdef _WIN32
	if (glGetProcAddress)
		glfunc = glGetProcAddress (name);
	else
		glfunc = GetProcAddress (handle, name);
# endif
#endif
	if (glfunc) {
		Sys_Printf ("found [0x%x]\n", (unsigned int) glfunc);
		return glfunc;
	} else {
		Sys_Printf ("not found\n");
		return NULL;
	}
}
